==> glibc-2.21.tar.bz2: glibc-2.21/sysdeps/ieee754/dbl-64/gamma_productf.c <==
/* Compute a product of X, X+1, ..., with an error estimate.
   Copyright (C) 2013-2015 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <math.h>
#include <math_private.h>
#include <float.h>

/* Compute the product of X + X_EPS, X + X_EPS + 1, ..., X + X_EPS + N
   - 1, in the form R * (1 + *EPS) where the return value R is an
   approximation to the product and *EPS is set to indicate the
   approximate error in the return value.  X is such that all the
   values X + 1, ..., X + N - 1 are exactly representable, and X_EPS /
   X is small enough that factors quadratic in it can be
   neglected.  */

float
__gamma_productf (float x, float x_eps, int n, float *eps)
{
  double x_full = (double) x + (double) x_eps;
  double ret = x_full;
  for (int i = 1; i < n; i++)
    ret *= x_full + i;

#if FLT_EVAL_METHOD != 0
  volatile
#endif
  float fret = ret;
  *eps = (ret - fret) / fret;

  return fret;
}

==> glibc-2.37.tar.bz2: glibc-2.37/sysdeps/ieee754/dbl-64/gamma_productf.c <==
/* Compute a product of X, X+1, ..., with an error estimate.
   Copyright (C) 2013-2023 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <https://www.gnu.org/licenses/>.  */

#include <math.h>
#include <math-narrow-eval.h>
#include <math_private.h>
#include <float.h>

/* Compute the product of X + X_EPS, X + X_EPS + 1, ..., X + X_EPS + N
   - 1, in the form R * (1 + *EPS) where the return value R is an
   approximation to the product and *EPS is set to indicate the
   approximate error in the return value.  X is such that all the
   values X + 1, ..., X + N - 1 are exactly representable, and X_EPS /
   X is small enough that factors quadratic in it can be
   neglected.  */

float
__gamma_productf (float x, float x_eps, int n, float *eps)
{
  double x_full = (double) x + (double) x_eps;
  double ret = x_full;
  for (int i = 1; i < n; i++)
    ret *= x_full + i;

  float fret = math_narrow_eval ((float) ret);
  *eps = (ret - fret) / fret;

  return fret;
}
